

from pathlib import Path

import climage
from rich.layout import Layout
from rich.padding import Padding
from rich.panel import Panel
from rich.table import Table
from rich.text import Text

from tvpy.console import cls
from tvpy.tv_json import load_tvpy
from tvpy.util import all_episodes, existing_episodes, last_episode_to_air


def tv_info(folder):
    try:
        info = load_tvpy(folder)
        poster = Path(folder) / '.poster.jpg'

        episodes = all_episodes(info)
        ss, es = zip(*episodes)
        max_s = max(ss)
        max_e = max(es)
        mat = [[''] * max_e for _ in range(max_s)]

        last_e = last_episode_to_air(info)

        for s, e in episodes:
            mat[s-1][e-1] = '[err]x' if (s, e) <= last_e else ':hourglass:'

        for s, e in existing_episodes(folder):
            mat[s-1][e-1] = '[success]v'

        POSTER_WIDTH = 38
        poster = Text.from_ansi(climage.convert(
            poster,
            is_unicode=True,
            width=POSTER_WIDTH))

        info = Panel(
            title=info["name"],
            renderable='\n'.join([
                f'Rating: {info["rating"]}:star:',
                f'Genres: {", ".join([g["name"] for g in info["genres"]])}']))

        episode_table = Table(
            '',
            title='[success]v[/success]: existing  [err]x[/err] missing  :hourglass: NA',
            title_style='gray',
            title_justify='left')

        for i in range(len(mat[0])):
            episode_table.add_column(f'E{i+1:02}', justify='center')

        for i, row in enumerate(mat):
            episode_table.add_row(f'S{i+1:02}', *row)

        layout = Layout()
        layout.split_column(
            Layout(' ', size=2),
            Layout(name='content'))

        layout['content'].split_column(
            Layout(name='up'),
            Padding(episode_table, (0, 2)))

        layout['up'].split_row(
            Layout(
                Padding(poster, 1),
                name='up',
                size=POSTER_WIDTH + 2),
            Padding(info, 1))

        cls.print(layout)

    except Exception as e:
        print(e)
        cls.print('[red]ERROR')



from tvpy.console import cls
from tvpy.tv_down import tv_down
from tvpy.tv_json import tv_json
from tvpy.tv_klyn import tv_klyn
from tvpy.tv_renm import tv_renm
from tvpy.tv_subs import tv_subs

logo = (r'''
[yellow2]      ___           ___           ___           ___     
[yellow2]     /\  \         /\__\         /\  \         |\__\    
[yellow2]     \:\  \       /:/  /        /::\  \        |:|  |   
[yellow2]      \:\  \     /:/  /        /:/\:\  \       |:|  |   
[yellow2]      /::\  \   /:/__/  ___   /::\~\:\  \      |:|__|__ 
[yellow2]     /:/\:\__\  |:|  | /\__\ /:/\:\ \:\__\     /::::\__\
[yellow2]    /:/  \/__/  |:|  |/:/  / \/__\:\/:/  /    /:/~~/~   
[yellow2]   /:/  /       |:|__/:/  /       \::/  /    /:/  /     
[yellow2]   \/__/         \::::/__/         \/__/     \/__/      
[yellow2]                  ~~~~                                  
[yellow2]                                                        
''')


def tvpy(folder):
    def sep(title):
        cls.print()
        cls.print(f'[dim orchid1]{title}')

    cls.print(logo)

    sep('Generating .tvpy.json')
    tv_json(folder)

    sep('Downloading episodes')
    tv_down(folder)

    sep('Renaming files')
    tv_renm(folder)

    sep('Downloading subtitles')
    tv_subs(folder)

    sep('Removing unused files')
    tv_klyn(folder)

    sep('Renaming files')
    tv_renm(folder)

    cls.save_svg('demo.svg')



import zipfile
from pathlib import Path

import requests
from PTN import parse
from rich.status import Status

from tvpy.console import cls
from tvpy.tv_json import load_tvpy
from tvpy.util import done, existing_episodes, files_subs


def list_available_subs(imdb_id, season, episode):
    url = f'https://wizdom.xyz/api/releases/{imdb_id}'
    res = requests.get(url).json()
    subs = res['subs']
    subs = subs[f'{season}'][f'{episode}']
    return subs


def select_sub(subs, encoder='NTB'):
    key = 'release_group'
    for sub in subs:
        if key in sub and sub[key] == encoder:
            return sub

    return subs[0]


def down_sub(sub_id, out_zip):
    url = f'https://wizdom.xyz/api/files/sub/{sub_id}'
    with open(out_zip, 'wb') as out:
        out.write(requests.get(url).content)


def existing_subs(folder):
    res = [parse(f.name) for f in files_subs(folder)]
    return {(e['season'], e['episode']) for e in res}


def tv_subs(folder):
    missing_subs = existing_episodes(folder) - existing_subs(folder)
    info = load_tvpy(folder)
    imdb_id = info['imdb_id']
    for s, e in missing_subs:
        name = f'{info["name"]} S{s:02}E{e:02}'
        with Status(f'[info]Searching...', console=cls) as status:
            try:
                subs = list_available_subs(imdb_id, s, e)
                sub = select_sub(subs)
                sub_version = sub['version']
                sub_id = sub['id']

                status.update('[info]Downloading...')
                zip_file = Path(folder) / f'{sub_version}.zip'
                down_sub(sub_id, zip_file)

                status.update('[info]Extracting...')
                with zipfile.ZipFile(zip_file) as z:
                    z.extractall(folder)

                status.stop()
                cls.print(f':clapper: [success]{name}')
            except:
                status.stop()
                cls.print(f'[err]Error:[/err] Could not find subtitles for {name}')

    done()



from pathlib import Path
from time import sleep
from typing import List, Tuple

import libtorrent as lt
import rich.status
from rich.progress import (BarColumn, Progress, TaskID, TaskProgressColumn,
                           TextColumn, TimeRemainingColumn)

from tvpy.console import cls
from tvpy.lt import Handler
from tvpy.torrent import torrents
from tvpy.tv_json import load_tvpy
from tvpy.tv_renm import file_name
from tvpy.util import done, missing_episodes


def q(folder, season: int, episode: int):
    name = Path(folder).name.replace('.', ' ')

    return f'{name} S{season:02}E{episode:02}'


def down(magnets, down_folder):
    s = lt.session()

    with Progress(
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TaskProgressColumn(),
            TimeRemainingColumn(),
            TextColumn('[dim cyan]Size: {task.fields[size]}MB'),
            TextColumn('[green]Down: {task.fields[down]}kB/s'),
            TextColumn('[blue]Up: {task.fields[up]}kB/s'),
            TextColumn('[orange1]Peers: {task.fields[peers]}')) as progress:

        tasks: List[Tuple[Handler, TaskID]] = []
        for name, link in magnets:
            params = lt.parse_magnet_uri(link)
            params.save_path = down_folder

            h = s.add_torrent(params)
            task = progress.add_task(f'[green]{name}', total=1, size='', down='', up='', peers='')
            tasks.append((h, task))

        while not progress.finished:
            for h, task in tasks:
                status = h.status()
                progress.update(
                    task,
                    completed=status.progress,
                    size=f'{status.total_done / 1048576:.1f}',
                    down=f'{status.download_rate / 1000:.1f}',
                    up=f'{status.upload_rate / 1000:.1f}',
                    peers=status.num_peers)

            sleep(1)


def tv_down(folder):
    try:
        tvpy = load_tvpy(folder)

        magnets = []
        with rich.status.Status('', console=cls) as status:
            for s, e in missing_episodes(folder, tvpy):
                name = file_name(tvpy, s, e)
                magnet = Path(folder) / f'{name}.magnet'
                if magnet.exists():
                    with open(magnet, 'r') as f:
                        magnet_link = f.read()
                else:
                    query = q(folder, s, e)
                    status.update(f'[info]Searching for {query}')
                    res = torrents.search(query)
                    items = res['items']
                    if not items:
                        cls.print(f'[warn]Could not find torrent for {query}')
                        continue

                    link = items[0]['link']
                    info = torrents.info(link)

                    magnet_link = info['magnetLink']
                    with open(magnet, 'w') as f:
                        f.write(magnet_link)

                magnets.append((name, magnet_link))

        if magnets:
            down(magnets, folder)

    except KeyboardInterrupt:
        pass

    done()



from itertools import chain
from pathlib import Path

from PTN import parse

from tvpy.console import cls


def load_key():
    return '7bfa2260d938bb3881e0dd89c47a6021'
    # with open('key.txt', 'r') as f:
    #     return f.read().strip()


def folders(root):
    return [x for x in Path(root).iterdir() if x.is_dir()]


def tvpys(folder):
    return Path(folder).rglob('.tvpy.json')


def files(folder, patterns=['*.mkv', '*.avi', '*.mp4', '*.srt']):
    return chain(*[Path(folder).glob(e) for e in patterns])


def files_r(folder, patterns=['*.mkv', '*.avi', '*.mp4', '*.srt']):
    return chain(*[Path(folder).rglob(e) for e in patterns])


def files_media(root):
    return files(root, patterns=['*.mkv', '*.avi', '*.mp4'])


def files_subs(root):
    return files(root, patterns=['*.srt'])


def files_media_r(root):
    return files_r(root, patterns=['*.mkv', '*.avi', '*.mp4'])


def files_subs_r(root):
    return files_r(root, patterns=['*.srt'])


def existing_episodes(folder):
    res = [parse(f.name) for f in files_media(folder)]
    return {(e['season'], e['episode']) for e in res}


def all_episodes(info):
    episodes = {
        (s['season_number'], i)
        for s in info['seasons']
        for i in range(1, s['episode_count'] + 1)}

    return sorted({(s, e) for s, e in episodes if s > 0})


def last_episode_to_air(info):
    res = info['last_episode_to_air']
    return (res['season_number'], res['episode_number'])


def on_air_episodes(info):
    return {
        e for e in all_episodes(info)
        if e <= last_episode_to_air(info)}


def missing_episodes(folder, tvpy, k=3):
    episodes = sorted(on_air_episodes(tvpy) - existing_episodes(folder))
    return episodes[:k]


def done():
    cls.print(f'[green]Done :+1:')



from fire import Fire


def tv_json():
    from tvpy.tv_json import tv_json
    Fire(tv_json)


def tv_down():
    from tvpy.tv_down import tv_down
    Fire(tv_down)


def tv_subs():
    from tvpy.tv_subs import tv_subs
    Fire(tv_subs)


def tv_renm():
    from tvpy.tv_renm import tv_renm
    Fire(tv_renm)


def tv_klyn():
    from tvpy.tv_klyn import tv_klyn
    Fire(tv_klyn)


def tv_info():
    from tvpy.tv_info import tv_info
    Fire(tv_info)


def tv_html():
    from tvpy.tv_html import tv_html
    Fire(tv_html)


def tvpy():
    from tvpy.tvpy import tvpy
    Fire(tvpy)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tvpy']

package_data = \
{'': ['*']}

install_requires = \
['1337x>=1.2.3,<2.0.0',
 'Pillow>=9.2.0,<10.0.0',
 'beautifulsoup4>=4.11.1,<5.0.0',
 'climage>=0.1.3,<0.2.0',
 'fire>=0.4.0,<0.5.0',
 'fs.smbfs>=1.0.5,<2.0.0',
 'libtorrent>=2.0.7,<3.0.0',
 'parse-torrent-title>=2.4,<3.0',
 'pyright>=1.1.273,<2.0.0',
 'requests>=2.28.1,<3.0.0',
 'rich>=12.5.1,<13.0.0',
 'toml>=0.10.2,<0.11.0',
 'tqdm>=4.64.0,<5.0.0',
 'websockets>=10.3,<11.0']

entry_points = \
{'console_scripts': ['tv-down = tvpy.main:tv_down',
                     'tv-html = tvpy.main:tv_html',
                     'tv-info = tvpy.main:tv_info',
                     'tv-json = tvpy.main:tv_json',
                     'tv-klyn = tvpy.main:tv_klyn',
                     'tv-renm = tvpy.main:tv_renm',
                     'tv-subs = tvpy.main:tv_subs',
                     'tvpy = tvpy.main:tvpy']}

setup_kwargs = {
    'name': 'tvpy',
    'version': '0.0.13',
    'description': '📺 TvPy',
    'long_description': '# 📺 TvPy \nCommand line tv show manager.\n\n[![asciicast](https://asciinema.org/a/hQeLoj8lYcGtJvErlTWifdmfo.svg)](https://asciinema.org/a/hQeLoj8lYcGtJvErlTWifdmfo)\n\n## Installation\n```shell\n> pip install tvpy\n```\n\n<!-- ## Get an API Key\nYou need to get an API key from [TMDB](https://www.themoviedb.org/settings/api) and save it as `key.txt` in your working directory. -->\n\n## Usage\n```shell\n> mkdir Carnival.Row \n> tvpy Carnival.Row \n```\n\n## Other commands\n\nDownload information from TMDB:\n```shell\n> mkdir Carnival.Row \n> tv-json Carnival.Row\n```\n\nDisplay information about a tv show:\n```shell\n> mkdir Carnival.Row \n> tv-info Carnival.Row\n```\n\nDownload a tv show:\n```shell\n> mkdir Carnival.Row \n> tv-down Carnival.Row\n```\n\nDownload (Hebrew) subtitles for a tv show:\n```shell\n> mkdir Carnival.Row \n> tv-subs Carnival.Row\n```\n\nRename files to match the pattern `<title>.S<season>E<episode>.<ext>`\n```shell\n> mkdir Carnival.Row \n> tv-renm Carnival.Row\n```\n\n| :exclamation:  Danger   |\n|-------------------------|\n\nRemove unused files\n```shell\n> mkdir Carnival.Row \n> tv-klyn Carnival.Row\n```\n\n\n',
    'author': 'Gilad Kutiel',
    'author_email': 'gilad.kutiel@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/gkutiel/tvpy/tree/master',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
